@@ -1,12 +1,13 @@
 # coding=utf-8
 
-from interlap import InterLap
-from itertools import chain
-from collections import defaultdict
-from typing import List, Optional, Union, Type
-import re
 import io
+import re
+from collections import defaultdict
+from itertools import chain
+from typing import List, Optional, Type, Union
+
 from cytoolz import memoize
+from interlap import InterLap
 
 
 class Feature:
@@ -14,37 +15,47 @@ class Feature:
     A class of genomic features defined by a half-open interval. Locations are 0-based.
     """
 
-    DUMMY_NAME = '.'
+    DUMMY_NAME = "."
 
     @staticmethod
     def from_string(line, transform=lambda x: x):
-        columns = re.split('[\t ]+', line.rstrip())
+        columns = re.split("[\t ]+", line.rstrip())
 
         assert (
             len(columns) >= 3
         ), "Invalid BED file supplied: at least three columns are expected. Please, check carefully that the correct input type was selected."
-        assert re.match("[0-9]+", columns[1]), "Invalid BED file supplied: second column must contain integers."
-        assert re.match("[0-9]+", columns[2]), "Invalid BED file supplied: third column must contain integers."
+        assert re.match(
+            "[0-9]+", columns[1]
+        ), "Invalid BED file supplied: second column must contain integers."
+        assert re.match(
+            "[0-9]+", columns[2]
+        ), "Invalid BED file supplied: third column must contain integers."
 
         name = Feature.DUMMY_NAME if len(columns) == 3 else transform(columns[3])
 
         try:
-            score = float(re.sub(',', '.', columns[4])) if len(columns) >= 5 else None
+            score = float(re.sub(",", ".", columns[4])) if len(columns) >= 5 else None
         except ValueError:
-            raise AssertionError("Invalid BED file supplied: fifth column must contain floating point numbers (score).")
+            raise AssertionError(
+                "Invalid BED file supplied: fifth column must contain floating point numbers (score)."
+            )
 
         strand = columns[5] if len(columns) >= 6 else None
 
         assert not strand or strand in (
-            '+',
-            '-',
-            '.',
-            '?',
+            "+",
+            "-",
+            ".",
+            "?",
         ), "Invalid BED file supplied: sixth column must contain strand (+/-/?)."
 
-        return Feature(columns[0], int(columns[1]), int(columns[2]), name, score, strand)
+        return Feature(
+            columns[0], int(columns[1]), int(columns[2]), name, score, strand
+        )
 
-    def __init__(self, chromosome, start, end, name=DUMMY_NAME, score=None, strand=None):
+    def __init__(
+        self, chromosome, start, end, name=DUMMY_NAME, score=None, strand=None
+    ):
         assert chromosome.strip() != ""
         assert end >= start
         assert name.strip() != ""
@@ -67,7 +78,9 @@ class Feature:
         return "Feature({})".format(str(self).replace("\t", ","))
 
     def __str__(self):
-        r = "{0:s}\t{1:d}\t{2:d}\t{3:s}".format(self.chromosome, self.interval[0], self.interval[1], self.name)
+        r = "{0:s}\t{1:d}\t{2:d}\t{3:s}".format(
+            self.chromosome, self.interval[0], self.interval[1], self.name
+        )
 
         if self.score and self.strand:
             r += "\t{0:f}\t{1:s}".format(self.score, self.strand)
@@ -99,7 +112,9 @@ class Feature:
         if not self.has_overlap_with(other):
             return 0
 
-        return min(self.interval[1], other.interval[1]) - max(self.interval[0], other.interval[0])
+        return min(self.interval[1], other.interval[1]) - max(
+            self.interval[0], other.interval[0]
+        )
 
 
 class FeatureSeq(object):
@@ -123,7 +138,7 @@ class FeatureSeq(object):
         else:
 
             def _feature_iterator():
-                with open(file, 'r') as f:
+                with open(file, "r") as f:
                     for line in f:
                         yield Feature.from_string(line, transform)
 
@@ -138,7 +153,10 @@ class FeatureSeq(object):
             self.chromosome2tree[feature.chromosome].add(
                 (
                     *feature.interval,
-                    {FeatureSeq.NAME_ATTRIBUTE: feature.name, FeatureSeq.SCORE_ATTRIBUTE: feature.score},
+                    {
+                        FeatureSeq.NAME_ATTRIBUTE: feature.name,
+                        FeatureSeq.SCORE_ATTRIBUTE: feature.score,
+                    },
                 )
             )
             self.name2features[feature.name].append(feature)
@@ -153,7 +171,7 @@ class FeatureSeq(object):
     def __str__(self):
         return "\n".join(map(str, self))
 
-    def get(self, name: str) -> 'FeatureSeq':
+    def get(self, name: str) -> "FeatureSeq":
         # Return a new list as a defensive copy mechanism.
         return FeatureSeq(self.name2features[name])
 
@@ -172,9 +190,17 @@ class FeatureSeq(object):
             if len(overlap_feature) == 0:
                 overlap_fraction_relative_to_overlap_feature = 0.0
             else:
-                overlap_fraction_relative_to_overlap_feature = overlap_in_bp / len(overlap_feature)
+                overlap_fraction_relative_to_overlap_feature = overlap_in_bp / len(
+                    overlap_feature
+                )
 
-            return max(overlap_fraction_relative_to_feature, overlap_fraction_relative_to_overlap_feature) >= fraction
+            return (
+                max(
+                    overlap_fraction_relative_to_feature,
+                    overlap_fraction_relative_to_overlap_feature,
+                )
+                >= fraction
+            )
 
         def toFeature(interval):
             return Feature(
@@ -188,11 +214,18 @@ class FeatureSeq(object):
         return list(
             filter(
                 filter4Fraction,
-                map(toFeature, self.chromosome2tree.get(feature.chromosome, InterLap()).find(feature.interval)),
+                map(
+                    toFeature,
+                    self.chromosome2tree.get(feature.chromosome, InterLap()).find(
+                        feature.interval
+                    ),
+                ),
             )
         )
 
-    def intersection(self, other: 'FeatureSeq', fraction: Optional[float] = None) -> 'FeatureSeq':
+    def intersection(
+        self, other: "FeatureSeq", fraction: Optional[float] = None
+    ) -> "FeatureSeq":
         def _feature_iterator(self, other):
             for feature1 in other:
                 for feature2 in self.find(feature1, fraction):
